--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: LGPL v2.1
File: HeadCount.lua
File description: Core  application logic / event handling
]]

local MAJOR_VERSION = "HeadCount-1.0"
local MINOR_VERSION = tonumber(("$Revision: 70808 $"):match("%d+"))

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

HeadCount = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
local HeadCount = HeadCount

HeadCount:RegisterDB("HeadCountDB", "HeadCountDBPC")

local raidTracker

-- Gets the raid tracker.
-- @return object Returns the raid tracker.
function HeadCount:getRaidTracker() 
	return raidTracker
end

-- Automatically called when the  addon is initialized
function HeadCount:OnInitialize()
	self:RegisterDefaults("profile", self:getDefaultOptions())
	self:RegisterChatCommand(L["console.commands"], self:getOptions())
	self.OnMenuRequest = self:getOptions()	-- fubar options menu
	
	self:LogInformation(string.format(L["info.initialization.complete"], L["product.name"], L["product.version.major"], L["product.version.minor"]))
	self:LogInformation(string.format(L["product.usage"], L["product.name"], L["product.version.major"], L["product.version.minor"])) 
end

-- Automatically called when the  addon is enabled
function HeadCount:OnEnable()
	self:RegisterSelfEvents()	-- register the main events

	if (raidTracker) then
		raidTracker:recover()	-- process recovery for the raid tracker		
		HeadCount:raidUpdate()
	else
		RequestRaidInfo()
	end
	
	self:LogDebug(string.format(L["debug.mod.enable"], L["product.name"], L["product.version.major"], L["product.version.minor"]))			
end

-- Automatically called when the  addon is disabled
function HeadCount:OnDisable()
	HeadCount:raidUpdate()	-- final update
	
	self:LogDebug(string.format(L["debug.mod.disable"], L["product.name"], L["product.version.major"], L["product.version.minor"]))		
end

-- Register self events
function HeadCount:RegisterSelfEvents()
	HeadCount:RegisterEvent("RAID_ROSTER_UPDATE", "raidUpdate")
	HeadCount:RegisterEvent("ZONE_CHANGED_NEW_AREA", "zoneChange") 
	HeadCount:RegisterEvent("CHAT_MSG_LOOT", "lootUpdate") 
	HeadCount:RegisterEvent("UPDATE_INSTANCE_INFO", "updateInstanceInfo")
	HeadCount:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "combatUpdate")
end

-- Automatically called when the UPDATE_INSTANCE_INFO event is triggered
function HeadCount:updateInstanceInfo() 
	if (not raidTracker) then
		HeadCount:UnregisterEvent("UPDATE_INSTANCE_INFO")
		
		local raidListWrapper = HeadCount:GetRaidListWrapper() 	
		local exclusionListWrapper = HeadCount:GetExclusionListWrapper()
		raidTracker = self.RaidTracker:new(raidListWrapper, exclusionListWrapper)	-- instantiate a new raid tracker 			
		raidTracker:recover()	-- process recovery for the raid tracker		
		HeadCount:raidUpdate()
	end	
end

-- Automatically called when the CHAT_MSG_LOOT event is triggered
function HeadCount:lootUpdate(message) 
	if (raidTracker) then
		raidTracker:processLootUpdate(message, activityTime)
	
		HeadCount:raidUpdate()	-- update
	end
end

-- Automatically called when the ZONE_CHANGED_NEW_AREA event is triggered
function HeadCount:zoneChange() 
	if (raidTracker) then
		local isZoneChanged = raidTracker:processZoneChange() 
	
		if (isZoneChanged) then 
			raidTracker:saveRaidListWrapper()	-- save the raid list
		end
	
		HeadCount:raidUpdate() -- update
	end
end	

-- Automatically called when the COMBAT_LOG_EVENT_UNFILTERED event is triggered
function HeadCount:combatUpdate(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags)
	if (event == "UNIT_DIED") then
		--HeadCount:LogInformation(destGUID .. " " .. destName .. " " .. destFlags)
	end
end

-- Automatically called when the  RAID_ROSTER_UPDATE event is triggered
function HeadCount:raidUpdate()
	if (raidTracker) then
		local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
		local activityTime = AceLibrary("HeadCountTime-1.0"):new({ ["utcDateTimeInSeconds"] = utcDateTimeInSeconds })
		
		if (UnitInRaid("player")) then 
			-- player is in a raid
			local currentRaid, isNewRaid = raidTracker:retrieveCurrentRaid(activityTime)	-- retrieve the current raid or create a new raid 
			if (isNewRaid) then
				-- new raid, disable all modal frames immediately
				DisableModalFrame() 
			end
			
			raidTracker:processZoneChange()
			
			-- Update each member in the raid group 
			local numberOfRaidMembers = GetNumRaidMembers()
			if (numberOfRaidMembers > 0) then
				-- active in a raid		

				local trackedPlayerList = { } 
				for i = 1,numberOfRaidMembers do
					local name, rank, subgroup, level, className, fileName, zone, isOnline, isDead, role, isML = GetRaidRosterInfo(i)
					
					-- self:debugRosterMember(name, rank, subgroup, level, className, fileName, zone, isOnline, isDead, role, isML)
					if (currentRaid:isPlayerPresent(name)) then 
						-- player is already present in the player list (existing player)
						local player = currentRaid:retrievePlayer(name)
						local timeDifference = HeadCount:computeTimeDifference(activityTime, player:getLastActivityTime())					
						local moveResult
						
						if (isOnline) then
							-- CURRENT: Player is online	
							local isPresentInRaidList = self:isRaidListGroup(subgroup)					
							local isPresentInWaitList = self:isWaitListGroup(subgroup)
							
							if (isPresentInRaidList) then
								-- CURRENT: Player is in the raid list.
								moveResult = player:moveToRaidList(activityTime)
								if (moveResult) then 
									currentRaid:moveToRaidGroup(player)
									self:LogDebug(string.format(L["debug.raid.update.modifymember.raidlist"], L["product.name"], L["product.version.major"], L["product.version.minor"], name))
								end
							elseif (isPresentInWaitList) then
								-- CURRENT: Player is in the wait list.
								moveResult = player:moveToWaitList(activityTime)
								if (moveResult) then 
									currentRaid:moveToRaidGroup(player)
									self:LogDebug(string.format(L["debug.raid.update.modifymember.waitlist"], L["product.name"], L["product.version.major"], L["product.version.minor"], name)) 
								end
							else
								moveResult = player:moveToNoList(activityTime, true)
								if (moveResult) then 
									currentRaid:moveToNoGroup(player)
									self:LogDebug(string.format(L["debug.raid.update.modifymember.nolist"], L["product.name"], L["product.version.major"], L["product.version.minor"], name)) 
								end
							end
						else
							-- CURRENT: Player is offline 
							moveResult = player:moveToNoList(activityTime, false)
							if (moveResult) then 							
								currentRaid:moveToNoGroup(player)						
								self:LogDebug(string.format(L["debug.raid.update.modifymember.offline"], L["product.name"], L["product.version.major"], L["product.version.minor"], name))																										
							end
						end
						
						player:updateValues(i)	-- update player values if needed
					else
						-- CREATE NEW PLAYER
						-- player is NOT present in the player list (new player)
						local _, race = UnitRace("raid" .. i)
						local sex = UnitSex("raid" .. i)
						local guild = GetGuildInfo("raid" .. i)
						
						if (0 == level) then 
							level = UnitLevel("raid" .. i)
						end
						
						if (name) then 
							-- protect against phantom players
							local player = raidTracker:createPlayer(name, race, guild, sex, level, className, subgroup, isOnline, activityTime)			
							currentRaid:addPlayer(player)	-- add the player to the raid player list						
						end
					end	-- end raid member is existing present member or new member 
					
					-- track the player
					if (name) then 
						trackedPlayerList[name] = name	
					end
				end	-- end loop through all raid members
				
				-- TODO: Remove/cleanup members that are no longer in the raid.  
				currentRaid:processMissingPlayers(numberOfRaidMembers, trackedPlayerList, activityTime)
				
				-- Update the last activity time for the raid after member processing is fully complete
				HeadCount:LogDebug(string.format(L["debug.raid.update.activitytime"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(activityTime))) 	
				currentRaid:setLastActivityTime(activityTime)
				self:LogDebug(string.format(L["debug.raid.update.numberOfRaidMembers"], L["product.name"], L["product.version.major"], L["product.version.minor"], currentRaid:getNumberOfPlayers()))			
			else
				-- no raid active, cleanup any existing raids
				if (raidTracker:getIsCurrentRaidActive()) then
					-- current raid is active, end it
					HeadCount:LogDebug(string.format(L["debug.raid.update.activitytime"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(activityTime))) 					
					HeadCount:endCurrentRaid(activityTime)
					self:LogDebug(string.format(L["debug.raid.update.end"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(activityTime)))		
				end
			end	-- end determine number of raid members
		else
			-- cleanup any existing raids
			if (raidTracker:getIsCurrentRaidActive()) then
				-- current raid is active, end it 
				HeadCount:LogDebug(string.format(L["debug.raid.update.activitytime"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(activityTime))) 				
				HeadCount:endCurrentRaid(activityTime)
				self:LogDebug(string.format(L["debug.raid.update.end"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(activityTime)))					
			end
		end -- end determine if mod owner is in a raid or not
		
		raidTracker:saveRaidListWrapper()	-- save the raid list
		HeadCountFrame_Update()		-- update the UI
	end
end

-- Ends the current raid
-- @param currentTime The current time.
function HeadCount:endCurrentRaid(currentTime) 
	local currentRaid = raidTracker:retrieveMostRecentRaid()
	
	if (currentRaid) then	
		-- finalize the current raid
		currentRaid:finalize(currentTime)			
		raidTracker:setIsCurrentRaidActive(false)	
	end
	
	HeadCount:raidUpdate()
end

-- Remove a raid.
-- @param raidId The raid id.
function HeadCount:removeRaid(raidId)
	raidTracker:removeRaid(raidId)
	HeadCount:raidUpdate()
end

-- Remove all raids.
function HeadCount:removeAllRaids() 
	raidTracker:removeAllRaids()
	HeadCount:raidUpdate()
end

-- Remove a player from a raid.
-- @param raidId The raid id.
-- @param playerName The player name.
function HeadCount:removePlayer(raidId, playerName) 
	raidTracker:removePlayer(raidId, playerName)
	HeadCount:raidUpdate()
end

-- Remove loot from a raid
-- @param raidId The raid id.
-- @param lootId The loot id.
function HeadCount:removeLoot(raidId, lootId) 
	raidTracker:removeLoot(raidId, lootId) 
	HeadCount:raidUpdate()
end

-- LOGGING --
-- Logs a debug message for a roster member
function HeadCount:debugRosterMember(name, rank, subgroup, level, className, fileName, zone, isOnline, isDead, role, isML) 
	local memberString = "[name: %s, rank: %d, subgroup: %d, level: %s, class: %s, fileName: %s, zone: %s, isOnline: %s, isDead: %s, role: %s, isML: %s]"

	local isMemberOnline = HeadCount:convertBooleanToString(isOnline)		
	local isMemberDead = HeadCount:convertBooleanToString(isDead)		
	local isMasterLooter = HeadCount:convertBooleanToString(isML)
	local memberRole
	if (role) then
		memberRole = role
	else
		memberRole = L["none"]
	end
		
	self:LogDebug(string.format(memberString, name, rank, subgroup, level, className, fileName, zone, isMemberOnline, isMemberDead, memberRole, isMasterLooter))	
end

-- Logs a debug message to the default chat frame
-- @param message The message.
-- @red The red color for this message.
-- @green The green color for this message.
-- @blue The blue color for this message.
function HeadCount:LogDebug(message, red, green, blue)
	if (self.db.profile.isDebugEnabled) then
		DEFAULT_CHAT_FRAME:AddMessage(message, red, green, blue)
	end
end

-- Logs a information message to the default chat frame
-- @param message The message.
-- @red The red color for this message.
-- @green The green color for this message.
-- @blue The blue color for this message.
function HeadCount:LogInformation(message, red, green, blue)
	DEFAULT_CHAT_FRAME:AddMessage(message, red, green, blue)
end

-- Logs a information message to the default chat frame with default colors
-- @param message The message.
function HeadCount:LogInformation(message)
	DEFAULT_CHAT_FRAME:AddMessage(message, 1.0, 1.0, 1.0)
end

-- Logs a warning message to the default chat frame with default colors
-- @param message The message.
function HeadCount:LogWarning(message)
	DEFAULT_CHAT_FRAME:AddMessage(message, 1.0, 1.0, 1.0)
end

-- Logs a message to the error chat frame
-- @param message The message.
-- @red The red color for this message.
-- @green The green color for this message.
-- @blue The blue color for this message.
function HeadCount:LogError(message, red, green, blue)
	UIErrorsFrame:AddMessage(message, red, green, blue)
end

----------------------------
--FuBar
----------------------------
HeadCount.hasNoText = true
HeadCount.hasIcon = "Interface\\Icons\\INV_Misc_Bone_HumanSkull_01"
HeadCount.hasNoColor = true
HeadCount.defaultPosition = "CENTER"
HeadCount.defaultMinimapPosition = 285
HeadCount.clickableTooltip = false
HeadCount.tooltipHiddenWhenEmpty = true
HeadCount.independentProfile = true
HeadCount.hideWithoutStandby = true
HeadCount.cannotDetachTooltip = true
HeadCount.blizzardTooltip = true

-- Minimap hover tooltip display
function HeadCount:OnTooltipUpdate()
	GameTooltip:AddLine("|cFF9999FF" .. L["product.name"])
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["minimap.frame"], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["minimap.configuration"], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["minimap.button.rotate"], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["minimap.button.drag"], 0.2, 1, 0.2)
end 

-- Minimap click display
function HeadCount:OnClick()
	if (UnitInRaid("player")) then
		HeadCount:raidUpdate()
	end
	
	ToggleUserInterface()
end
