--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: LGPL v2.1
File: RaidTracker.lua
File description: Tracks raids.
]]

local AceOO = AceLibrary("AceOO-2.0")
local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

HeadCount.RaidTracker = AceOO.Class()

HeadCount.RaidTracker.prototype.isCurrentRaidActive = nil	-- true if the current raid is active, false otherwise
HeadCount.RaidTracker.prototype.raidListWrapper = nil		-- the raid list wrapper
HeadCount.RaidTracker.prototype.exclusionListWrapper = nil	-- the exclusion list wrapper

function HeadCount.RaidTracker.prototype:init(raidListWrapper, exclusionListWrapper)
    HeadCount.RaidTracker.super.prototype.init(self)
	
	self.isCurrentRaidActive = false 
	
	if (raidListWrapper) then 
		self.raidListWrapper = raidListWrapper 
	else
		self.raidListWrapper = AceLibrary("HeadCountRaidListWrapper-1.0"):new({ ["raidList"] = { }, ["numberOfRaids"] = 0 })	-- create an online player, starting in no list
	end

	if (exclusionListWrapper) then 
		self.exclusionListWrapper = exclusionListWrapper 
	else
		self.exclusionListWrapper = AceLibrary("HeadCountExclusionListWrapper-1.0"):new({ ["exclusionList"] = { }, ["numberOfExcludedItems"] = 0 })	-- create an online player, starting in no list
	end
end

-- Saves the raid list to the db profile
function HeadCount.RaidTracker.prototype:saveRaidListWrapper() 
	HeadCount:SetRaidListWrapper(self.raidListWrapper)
end

-- Process recovery for the raid tracker
function HeadCount.RaidTracker.prototype:recover() 
	local raid = self:retrieveMostRecentRaid() 	
	if (raid) then 
		local isRaidFinalized = raid:getIsFinalized() 
		local isUnitInRaid = UnitInRaid("player")	
		
		local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
		local activityTime = AceLibrary("HeadCountTime-1.0"):new({ ["utcDateTimeInSeconds"] = utcDateTimeInSeconds })
		
		if (isUnitInRaid) then 
			-- owner is present in raid
			if (isRaidFinalized) then 
				-- most recent raid is finalized
				HeadCount:LogDebug(string.format(L["debug.raidtracker.setup.raid.final"], L["product.name"], L["product.version.major"], L["product.version.minor"]))
				self.isCurrentRaidActive = false				
			else
				-- most recent raid is NOT finalized
				HeadCount:LogDebug(string.format(L["debug.raidtracker.setup.raid.nofinal"], L["product.name"], L["product.version.major"], L["product.version.minor"]))
				raid:recover(activityTime)
				self.isCurrentRaidActive = true
			end
		else
			-- owner is NOT present in raid
			if (isRaidFinalized) then 
				-- most recent raid is finalized
				HeadCount:LogDebug(string.format(L["debug.raidtracker.setup.noraid.final"], L["product.name"], L["product.version.major"], L["product.version.minor"]))
				self.isCurrentRaidActive = false							
			else
				-- most recent raid is NOT finalized
				HeadCount:LogDebug(string.format(L["debug.raidtracker.setup.noraid.nofinal"], L["product.name"], L["product.version.major"], L["product.version.minor"]))
				raid:finalize(activityTime)
				self.isCurrentRaidActive = false						
			end		
		end		
	else
		--HeadCount:LogInformation("The most recent raid does not exist.")
	end	
end

-- Determines if the current raid is endable
-- @return boolean Returns true if the current raid can be ended, false otherwise.
function HeadCount.RaidTracker.prototype:isCurrentRaidEndable()
	local isEndable = false

	local numberOfRaids = self.raidListWrapper:getNumberOfRaids()
	if ((self.isCurrentRaidActive) and (numberOfRaids > 0)) then
		-- raids exist and there is a current raid active
		isEndable = true
	end	
	
	return isEndable
end

-- Remove a raid.
-- @param id The raid id.
function HeadCount.RaidTracker.prototype:removeRaid(id)
	local isPresent = self.raidListWrapper:isRaidPresent(id)

	if (isPresent) then 
		-- raid is present and raid
		local raid = self.raidListWrapper:getRaidById(id)
		if ((not raid:getIsFinalized()) and (self.isCurrentRaidActive)) then
			HeadCount:LogInformation("Removing raid: " .. id .. " (removing the current raid)")
			-- raid is not finalized, raid is active
			self.isCurrentRaidActive = false	-- no current raid			
		end
		
		self.raidListWrapper:removeRaid(id)
	end
end

-- Remove all raids.
function HeadCount.RaidTracker.prototype:removeAllRaids() 
	self.isCurrentRaidActive = false
	self.raidListWrapper:removeAll()
end

-- Removes a player from a given raid.
-- @param id The raid id.
-- @param playerName The player name.
function HeadCount.RaidTracker.prototype:removePlayer(id, playerName)
	local raid = self.raidListWrapper:getRaidById(id) 	
	if (raid) then
		-- raid exists
		raid:removePlayer(playerName)
	end
end

-- Removes loot from a given raid.
-- @param raidId The raid id.
-- @param lootId The loot id.
function HeadCount.RaidTracker.prototype:removeLoot(raidId, lootId) 
	local raid = self.raidListWrapper:getRaidById(raidId)
	if (raid) then 
		-- raid exists
		raid:removeLoot(lootId)
	end
end

-- Gets the current active raid status
-- @return boolean Returns true if the current raid is active, false otherwise.
function HeadCount.RaidTracker.prototype:getIsCurrentRaidActive()
	return self.isCurrentRaidActive
end

-- Sets the current active raid status
-- @param isCurrentRaidActive The current active raid status
function HeadCount.RaidTracker.prototype:setIsCurrentRaidActive(isCurrentRaidActive) 
	self.isCurrentRaidActive = isCurrentRaidActive
end

-- Retrieves the most recent raid.
-- @return object Returns the most recently added raid or nil if none exists.
function HeadCount.RaidTracker.prototype:retrieveMostRecentRaid() 	
	local raid = self.raidListWrapper:retrieveMostRecentRaid()
	
	return raid
end

-- Retrieves the current raid id.
-- @return number Returns the current raid id
function HeadCount.RaidTracker.prototype:retrieveCurrentRaidId()
	local raidId = self.raidListWrapper:retrieveMostRecentRaidId()

	return raidId
end

-- Retrieves the ordered raid list.
-- @return table Returns the ordered raid list
function HeadCount.RaidTracker.prototype:retrieveOrderedRaidList(isDescending)	 
	local orderedRaidList = self.raidListWrapper:retrieveOrderedRaidList(isDescending) 
	
	return orderedRaidList
end

-- Sets the current raid
-- @param raid The current raid.
function HeadCount.RaidTracker.prototype:addRaid(raid) 
	local raidStartTime = raid:retrieveStartingTime():getUTCDateTimeInSeconds()
	self.raidListWrapper:addRaid(raidStartTime, raid)
end

-- Gets the number of raids
-- @return number Returns the number of raids
function HeadCount.RaidTracker.prototype:getNumberOfRaids()
	return self.raidListWrapper:getNumberOfRaids()
end

-- Gets the raid list wrapper.
-- @return object Returns the raid list wrapper.
function HeadCount.RaidTracker.prototype:getRaidListWrapper()
	return self.raidListWrapper
end

-- Sets the exclusion list wrapper.
-- @param exclusionList The exclusion list wrapper.
function HeadCount.RaidTracker.prototype:setExclusionListWrapper(exclusionListWrapper)
	self.exclusionListWrapper = exclusionListWrapper
end

-- Gets a given raid by its id.
-- @param id The raid id.
-- @return object Returns the given raid or nil if none exists.
function HeadCount.RaidTracker.prototype:getRaidById(id) 
	return self.raidListWrapper:getRaidById(id)
end

-- Retrieve the current active raid or creates a new raid if no current raid exists.
-- @param activityTime The activity time.
-- @return object Returns the current raid.
function HeadCount.RaidTracker.prototype:retrieveCurrentRaid(activityTime) 
	local currentRaid = nil
	local isNewRaid = false
	
	if (self.isCurrentRaidActive) then
		-- a raid currently exists, process existing raid
		currentRaid = self:retrieveMostRecentRaid()
		isNewRaid = false
		HeadCount:LogDebug(string.format(L["debug.raid.update.continue"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(currentRaid:retrieveStartingTime()), HeadCount:getDateTimeAsString(currentRaid:getLastActivityTime())))
	else
		-- a raid does not already exist, create a new raid, then process it
		local args = {
			["playerList"] = {
				["raidgroup"] = { }, 
				["nogroup"] = { }, 
			}, 
			["lootList"] = { }, 
			["timeList"] = { }, 
			["raidTime"] = 0, 
			["lastActivityTime"] = activityTime, 
			["zone"] = nil, 
			["numberOfPlayers"] = 0, 
			["isFinalized"] = false
		}

		local timePair = AceLibrary("HeadCountTimePair-1.0"):new({ ["beginTime"] = nil, ["endTime"] = nil, ["note"] = nil })			
		timePair:setBeginTime(activityTime)
		table.insert(args["timeList"], timePair)	-- insert the first time pair (includes begin time with no end time)		
		
		currentRaid = AceLibrary("HeadCountRaid-1.0"):new(args)			
		self:setIsCurrentRaidActive(true)
		self:addRaid(currentRaid)
		isNewRaid = true
		HeadCount:LogInformation(string.format(L["info.newraid"], L["product.name"], L["product.version.major"], L["product.version.minor"], HeadCount:getDateTimeAsString(currentRaid:getLastActivityTime())))
	end
	
	return currentRaid, isNewRaid 
end

-- Process a zone change.
-- If the current raid has no raid zone assigned to it, set the raid zone name.
-- @return boolean Returns true if the zone changed and false otherwise.
function HeadCount.RaidTracker.prototype:processZoneChange() 
	local isZoneChanged = false

	local isPresentInInstance, instanceType = IsInInstance()	
	if ((isPresentInInstance) and (instanceType == "raid") and (self.isCurrentRaidActive)) then
		-- player is in a raid instance and a tracked raid is active
		local currentRaid = self:retrieveMostRecentRaid()
		local realZoneText = GetRealZoneText() 
		if (not currentRaid:getZone()) then
			-- current raid zone is not set
			if (L.raidInstances[realZoneText]) then
				currentRaid:setZone(L.raidInstances[realZoneText])
				isZoneChanged = true
			end
		end
	end
	
	return isZoneChanged
end

-- Processed a loot update. 
-- LOOT_ITEM: %s receives loot: %s.
-- LOOT_ITEM_MULTIPLE: %s receives loot: %sx%d.
-- LOOT_ITEM_SELF: You receive loot: %s.
-- LOOT_ITEM_SELF_MULTIPLE: You receive loot: %sx%d.
-- @param message The loot message.
function HeadCount.RaidTracker.prototype:processLootUpdate(message) 
	if (self.isCurrentRaidActive) then 
		-- current raid is active
		local playerName, item, quantity = self:processLootPattern(message) 
		if (playerName) then 
			-- message matched proper loot pattern 
			local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
			local activityTime = AceLibrary("HeadCountTime-1.0"):new({ ["utcDateTimeInSeconds"] = utcDateTimeInSeconds })			
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item) 
			
			local minimumLootQuality = L.itemQualityNumbers[HeadCount:GetMinimumLootQuality()] 
			local id = string.match(itemLink, "item:(%d+):")
			if ((itemRarity >= minimumLootQuality) and (not self:isExclusionItemPresent(id))) then 
				-- item dropped is greater than or equal to minimum loot quality and the item is not excluded, track it!  
				local realZoneText = GetRealZoneText()
				
				local itemId = AceLibrary("HeadCountLoot-1.0"):retrieveItemId(itemLink)	-- get the item id
				local loot = AceLibrary("HeadCountLoot-1.0"):new({ ["itemId"] = itemId, ["playerName"] = playerName, ["activityTime"] = activityTime, ["quantity"] = quantity, ["zone"] = realZoneText })
				local currentRaid = self:retrieveMostRecentRaid()
				currentRaid:addLoot(loot)				
			end
		end
	end
end

-- Processes a loot message pattern
-- @param message The loot message.
-- @return string, string, number  Returns the player name, item name, and quantity or nil if the message does not match any pattern
function HeadCount.RaidTracker.prototype:processLootPattern(message) 
	local playerName = nil
	local item = nil
	local quantity = nil
	
	local multipleLootSelfRegex = HeadCount:getMULTIPLE_LOOT_SELF_REGEX()
	local lootSelfRegex = HeadCount:getLOOT_SELF_REGEX()
	local multipleLootRegex = HeadCount:getMULTIPLE_LOOT_REGEX() 
	local lootRegex = HeadCount:getLOOT_REGEX() 	
	
	if (message:match(multipleLootSelfRegex)) then 
		item, quantity = message:match(multipleLootSelfRegex) 
		playerName = UnitName("player") 
		--HeadCount:LogInformation("MULTIPLE SELF LOOT: " .. playerName .. " received loot " .. item .. "x" .. quantity)
	elseif (message:match(lootSelfRegex)) then 
		item = message:match(lootSelfRegex) 
		playerName = UnitName("player") 
		quantity = 1
		--HeadCount:LogInformation("SELF LOOT: " .. playerName .. " received loot " .. item)		
	elseif (message:match(multipleLootRegex)) then 
		playerName, item, quantity = message:match(multipleLootRegex) 
		--HeadCount:LogInformation("MULTIPLE LOOT: " .. playerName .. " received loot " .. item .. "x" .. quantity)
	elseif (message:match(lootRegex)) then 
		playerName, item = message:match(lootRegex) 
		quantity = 1
		--HeadCount:LogInformation("LOOT: " .. playerName .. " received loot " .. item) 		
	else
		--HeadCount:LogInformation("The loot message did not match any pattern.")
	end	
	
	return playerName, item, quantity
end

-- Creates a player.
-- @param name The name.
-- @param race The race.
-- @param guild The guild name.
-- @param sex The sex.
-- @param level The player level.
-- @param className The class name.
-- @param subgroup The subgroup.
-- @param isOnline The online status.
-- @param activityTime The activity time.
-- @return object Returns a player or nil if none can be created.
function HeadCount.RaidTracker.prototype:createPlayer(name, race, guild, sex, level, className, subgroup, isOnline, activityTime) 
	local player = nil
	
	local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()	
	local activityTime = AceLibrary("HeadCountTime-1.0"):new({ ["utcDateTimeInSeconds"] = utcDateTimeInSeconds })		

	local timePair = AceLibrary("HeadCountTimePair-1.0"):new({ ["beginTime"] = nil, ["endTime"] = nil, ["note"] = nil })		
	timePair:setBeginTime(activityTime) 
	
	local args = {
		["name"] = name, 
		["className"] = className, 
		["race"] = race, 
		["guild"] = guild, 
		["sex"] = sex, 
		["level"] = level, 
		["isPresentInRaidList"] = false, 
		["isPresentInWaitList"] = false, 
		["isOnline"] = true, 
		["raidListTime"] = 0, 
		["waitListTime"] = 0, 
		["offlineTime"] = 0, 
		["timeList"] = { }, 
		["lastActivityTime"] = activityTime, 
		["isFinalized"] = false	
	}
	
	if (isOnline) then
		-- CURRENT: Player is online
		if (HeadCount:isRaidListGroup(subgroup)) then
			-- CURRENT: Player is in the raid list.
			args["isPresentInRaidList"] = true
			args["isPresentInWaitList"] = false
			args["isOnline"] = true			
			
			timePair:setNote(L["Raid list"])
			table.insert(args["timeList"], timePair)	-- insert the first time pair (includes begin time with no end time)			
			
			player = AceLibrary("HeadCountPlayer-1.0"):new(args)	-- create an online player, starting in the raid list
			HeadCount:LogDebug(string.format(L["debug.raid.update.addmember.raidlist"], L["product.name"], L["product.version.major"], L["product.version.minor"], name))
		elseif (HeadCount:isWaitListGroup(subgroup)) then		
			-- CURRENT: Player is in the wait list.
			args["isPresentInRaidList"] = false
			args["isPresentInWaitList"] = true
			args["isOnline"] = true			
			
			timePair:setNote(L["Wait list"])
			table.insert(args["timeList"], timePair)	-- insert the first time pair (includes begin time with no end time)			
			
			player = AceLibrary("HeadCountPlayer-1.0"):new(args)	-- create an online player, starting in the wait list
			HeadCount:LogDebug(string.format(L["debug.raid.update.addmember.waitlist"], L["product.name"], L["product.version.major"], L["product.version.minor"], name))														
		else
			-- CURRENT: Player is in no list.
			args["isPresentInRaidList"] = false
			args["isPresentInWaitList"] = false
			args["isOnline"] = true			
			
			timePair:setNote(L["No list"])
			table.insert(args["timeList"], timePair)	-- insert the first time pair (includes begin time with no end time)			
			
			player = AceLibrary("HeadCountPlayer-1.0"):new(args)	-- create an online player, starting in no list
			HeadCount:LogDebug(string.format(L["debug.raid.update.addmember.nolist"], L["product.name"], L["product.version.major"], L["product.version.minor"], name))
		end
	else
		-- CURRENT: Player is offline
		args["isPresentInRaidList"] = false
		args["isPresentInWaitList"] = false
		args["isOnline"] = false
		
		timePair:setNote(L["Offline"])
		table.insert(args["timeList"], timePair)	-- insert the first time pair (includes begin time with no end time)			
		
		player = AceLibrary("HeadCountPlayer-1.0"):new(args)	-- create an online player, starting in no list
		HeadCount:LogDebug(string.format(L["debug.raid.update.addmember.offline"], L["product.name"], L["product.version.major"], L["product.version.minor"], name))
	end
	
	return player
end

-- Gets the exclusion list wrapper.
-- @return object Returns the exclusion list wrapper.
function HeadCount.RaidTracker.prototype:getExclusionListWrapper()
	return self.exclusionListWrapper
end

-- Sets the raid list wrapper.
-- @param raidList The raid list wrapper.
function HeadCount.RaidTracker.prototype:setRaidListWrapper(raidListWrapper)
	self.raidListWrapper = raidListWrapper
end

-- Determines if an exclusion item is present
-- @param string The exclusion item.
-- @return boolean Returns true if the item is excluded and false otherwise.
function HeadCount.RaidTracker.prototype:isExclusionItemPresent(item) 
	return self.exclusionListWrapper:isItemPresent(item) 
end

-- Add an excluded item.
-- @param id The loot id.
-- @param item The excluded loot item.
function HeadCount.RaidTracker.prototype:addExclusionItem(id, loot) 
	self.exclusionListWrapper:addItem(id, loot)
end

-- Remove an excluded item.
-- @param id The excluded loot id.
function HeadCount.RaidTracker.prototype:removeExclusionItem(id) 
	self.exclusionListWrapper:removeItem(id)
end

-- Gets the number of excluded items
-- @return number Returns the number of excluded items
function HeadCount.RaidTracker.prototype:getNumberOfExcludedItems()
	return self.exclusionListWrapper:getNumberOfExcludedItems()
end

-- To String
-- @return string Returns the string description for this object.
function HeadCount.RaidTracker.prototype:ToString()
	return L["object.RaidTracker"]
end
